// BaseApplication.cpp
// Base application functionality for inheritnace.
#include "BaseApplication.h"


BaseApplication::BaseApplication()
{
	
}


BaseApplication::~BaseApplication()
{
	// Release the timer object.
	if (m_Timer)
	{
		delete m_Timer;
		m_Timer = 0;
	}

	// Release the Direct3D object.
	if (m_Direct3D)
	{
		delete m_Direct3D;
		m_Direct3D = 0;
	}
}

void BaseApplication::init(HINSTANCE hinstance, HWND hwnd, int screenWidth, int screenHeight, Input *in, INIReader* iniReader)
{
	bool vsync_enabled, fullscreen;
	float fov;
	AntiAliasingSetting antiAliasingSetting;
	std::string aaSetting;
	m_Input = in;
	m_hwnd = hwnd;
	m_screenWidth = screenWidth;
	m_screenHeight = screenHeight;

	// Read in the graphics settings from the settings ini
	// point the ini reader to the settings file
	iniReader->setFilename("../res/data/settings.ini");

	fov = stof(iniReader->getValue("fov", "settings"));
	vsync_enabled = (iniReader->getValue("vsync", "settings") == "enabled");
	fullscreen = (iniReader->getValue("fullscreen", "settings") == "enabled");
	m_resolutionWidth = std::stoi(iniReader->getValue("resolutionX", "settings"));
	m_resolutionHeight = std::stoi(iniReader->getValue("resolutionY", "settings"));

	aaSetting = iniReader->getValue("aaSetting", "settings");
	
	antiAliasingSetting = ReadAASetting(aaSetting);

	// Create the Direct3D object.
	m_Direct3D = D3D::instance();
	m_Direct3D->init(screenWidth, screenHeight, m_resolutionWidth, m_resolutionHeight, vsync_enabled, hwnd, fullscreen, antiAliasingSetting);
	
	// Initialize the Direct3D object.
	if (!m_Direct3D)
	{
		MessageBox(hwnd, (LPCSTR)"Could not initialize DirectX 11.", (LPCSTR)"Error", MB_OK);
		exit(EXIT_FAILURE);
	}

	// Create the timer object.
	m_Timer = new Timer();

	// Create the scene object
	m_Scene = new TestScene(m_Direct3D, m_hwnd, iniReader);
}

bool BaseApplication::Frame()
{
	// Check if the user pressed escape and wants to exit the application.
	if (m_Input->isKeyDown(VK_ESCAPE) == true)
	{
		return false;
	}

	// Update the system stats.
	m_Timer->Frame();

	// Do the frame input processing.
	HandleInput(m_Timer->GetTime());

	m_Scene->Update(m_Input, m_Timer->GetTime());

	return true;
}


void BaseApplication::HandleInput(float frameTime)
{

}

AntiAliasingSetting BaseApplication::ReadAASetting(std::string stringSetting)
{
	AntiAliasingSetting antiAliasingSetting;

	// read in the antialiasing setting then edit the shader settings file if necessary
	if (stringSetting == "fxaa")
	{
		EditShaderSettingsFile(1);
		antiAliasingSetting = AntiAliasingSetting::FXAA;
	}
	else if (stringSetting == "msaa2x")
	{
		EditShaderSettingsFile(2);
		antiAliasingSetting = AntiAliasingSetting::MSAA2X;
	}
	else if (stringSetting == "msaa4x")
	{
		EditShaderSettingsFile(4);
		antiAliasingSetting = AntiAliasingSetting::MSAA4X;
	}
	else if (stringSetting == "msaa8x")
	{
		EditShaderSettingsFile(8);
		antiAliasingSetting = AntiAliasingSetting::MSAA8X;
	}
	else if (stringSetting == "msaa16x")
	{
		EditShaderSettingsFile(16);
		antiAliasingSetting = AntiAliasingSetting::MSAA16X;
	}
	else if (stringSetting == "txaa")
	{
		EditShaderSettingsFile(1);
		antiAliasingSetting = AntiAliasingSetting::TXAA;
	}
	else
	{
		EditShaderSettingsFile(1);
		antiAliasingSetting = AntiAliasingSetting::DISABLED;
	}

	return antiAliasingSetting;
}

void BaseApplication::EditShaderSettingsFile(int sampleCount)
{
	ofstream settingsFile;

	settingsFile.open("../res/shaders/shader_settings.hlsli", ios::out | ios::trunc);

	string settingsString = "#define SAMPLE_COUNT " + std::to_string(sampleCount);
	settingsFile << settingsString;

	settingsFile.close();
}