#include "BaseScene.h"

ResourceContainer *ResourceContainer::m_containerInstance = 0;

BaseScene::BaseScene(D3D* d3d, HWND hwnd, INIReader* iniReader)
{
	m_deviceContext = d3d->GetDeviceContext();
	m_device = d3d->GetDevice();
	m_Direct3D = d3d;
	m_hwnd = hwnd;
	m_resourceContainer = ResourceContainer::instance();
}

BaseScene::BaseScene(BaseScene& copy)
{
}

BaseScene::~BaseScene()
{
	m_resourceContainer->ClearResources();
	for (auto object : m_objects)
	{
		delete object;
	}
	m_objects.clear();
}

SceneObject* BaseScene::GetObjectByName(const std::wstring& name)
{
	for (auto object : m_objects)
	{
		if (object->GetName() == name)
			return object;
	}
}

void BaseScene::InitRendererNoAA()
{
	BaseShader* shader;
	DeferredBuffers* buffers;

	// Create the g-buffer buffers
	buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetResolutionWidth(), m_Direct3D->GetResolutionHeight(), 2, 1);
	m_resourceContainer->AddBuffer(L"G_BUFFER", buffers);

	// Create the G-Buffer shader and add it to the resource manager
	shader = 0;
	shader = new PrePassGBufferShader(m_device, m_hwnd);
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"G_BUFFER"));
	shader->AddCamera(m_resourceContainer->GetCameraIndex(L"MAIN_CAMERA"));
	m_resourceContainer->AddShader(L"G_BUFFER_SHADER", shader);

	// Create the Mask shader and add it to the resource manager
	shader = 0;
	shader = new PrePassMaskShader(m_device, m_hwnd);
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"G_BUFFER"));
	m_resourceContainer->AddShader(L"MASK_SHADER", shader);

	// Create the light buffer
	buffers = 0;
	buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetResolutionWidth(), m_Direct3D->GetResolutionHeight(), 1, 1);

	// Create the variable that is used to turn shadows on and off
	FloatParameter* floatParam = new FloatParameter(1, NULL);
	floatParam->UpdateParameter(0, 0.0f);

	// Create the Light shader and add it to the resource manager
	shader = 0;
	shader = new PrePassLightShader(m_device, m_hwnd, false);
	shader->AddBuffers(m_resourceContainer->AddBuffer(L"LIGHT_BUFFER", buffers));
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"G_BUFFER"));
	shader->AddCamera(m_resourceContainer->GetCameraIndex(L"MAIN_CAMERA"));
	shader->AddFloat(m_resourceContainer->AddFloat(L"SHADOW_SWITCH", floatParam));
	m_resourceContainer->AddShader(L"LIGHT_SHADER", shader);

	// Create the Final shader and add it to the resource manager
	shader = 0;
	shader = new PrePassFinalShader(m_device, m_hwnd, false);
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"LIGHT_BUFFER"));
	shader->AddCamera(m_resourceContainer->GetCameraIndex(L"MAIN_CAMERA"));
	m_resourceContainer->AddShader(L"FINAL_SHADER", shader);
}

void BaseScene::InitRendererAA(int sampleCount)
{
	BaseShader* shader;
	DeferredBuffers* buffers;

	// Create the g-buffer buffers
	buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetResolutionWidth(), m_Direct3D->GetResolutionHeight(), 2, sampleCount);
	m_resourceContainer->AddBuffer(L"G_BUFFER", buffers);

	// Create the G-Buffer shader and add it to the resource manager
	shader = 0;
	shader = new PrePassGBufferShader(m_device, m_hwnd);
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"G_BUFFER"));
	shader->AddCamera(m_resourceContainer->GetCameraIndex(L"MAIN_CAMERA"));
	m_resourceContainer->AddShader(L"G_BUFFER_SHADER", shader);

	// Create the Mask shader and add it to the resource manager
	shader = 0;
	shader = new PrePassMaskShader(m_device, m_hwnd);
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"G_BUFFER"));
	m_resourceContainer->AddShader(L"MASK_SHADER", shader);

	// Create the light buffer
	buffers = 0;
	buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetResolutionWidth(), m_Direct3D->GetResolutionHeight(), 1, sampleCount);

	// Create the variable that is used to turn shadows on and off
	FloatParameter* floatParam = new FloatParameter(1, NULL);
	floatParam->UpdateParameter(0, 0.0f);

	// Create the Light shader and add it to the resource manager
	shader = 0;
	shader = new PrePassLightShader(m_device, m_hwnd, true);
	shader->AddBuffers(m_resourceContainer->AddBuffer(L"LIGHT_BUFFER", buffers));
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"G_BUFFER"));
	shader->AddCamera(m_resourceContainer->GetCameraIndex(L"MAIN_CAMERA"));
	shader->AddFloat(m_resourceContainer->AddFloat(L"SHADOW_SWITCH", floatParam));
	m_resourceContainer->AddShader(L"LIGHT_SHADER", shader);

	// Create the Final shader and add it to the resource manager
	shader = 0;
	shader = new PrePassFinalShader(m_device, m_hwnd, true);
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"LIGHT_BUFFER"));
	shader->AddCamera(m_resourceContainer->GetCameraIndex(L"MAIN_CAMERA"));
	m_resourceContainer->AddShader(L"FINAL_SHADER", shader);
}

void BaseScene::InitFXAA()
{
	DeferredBuffers* buffers;
	BaseShader* shader;

	// create the output texture for the fxaa pass
	buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetResolutionWidth(), m_Direct3D->GetResolutionHeight(), 1, 1);

	// create the fxaa shader
	shader = new FxaaShader(m_device, m_hwnd);
	shader->AddBuffers(m_resourceContainer->AddBuffer(L"FXAA_OUTPUT_BUFFER", buffers));
	m_resourceContainer->AddShader(L"FXAA_SHADER", shader);

	// Create the buffer copying shader for this effect
	shader = 0;
	shader = new BufferScalingShader(m_device, m_hwnd);
	reinterpret_cast<BufferScalingShader*>(shader)->SetBuffers(buffers->GetShaderResourceView(0), m_Direct3D->GetIntermediateBufferRenderTarget());
	m_resourceContainer->AddShader(L"FXAA_BUFFER_SCALING_SHADER", shader);
}

void BaseScene::InitDepthShader()
{
	DeferredBuffers* buffers;
	BaseShader* shader;

	// Create the depth storage buffer
	buffers = 0;
	buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetResolutionWidth(), m_Direct3D->GetResolutionHeight(), 1, 1);
	m_resourceContainer->AddBuffer(L"DEPTH_BUFFER", buffers);

	// Create the depth shader
	shader = 0;
	shader = new DepthShader(m_device, m_hwnd);
	shader->AddCamera(m_resourceContainer->GetCameraIndex(L"MAIN_CAMERA"));
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"DEPTH_BUFFER"));
	m_resourceContainer->AddShader(L"DEPTH_SHADER", shader);
}

void BaseScene::InitShadowMapShader()
{
	BaseShader* shader;

	// Create the shadow map shader
	shader = 0;
	shader = new ShadowMapShader(m_device, m_hwnd);
	m_resourceContainer->AddShader(L"SHADOW_MAP_SHADER", shader);

}

void BaseScene::InitAmbientOcclusion()
{
	// Create the random vector map
	D3D11_SUBRESOURCE_DATA initData;
	initData.SysMemPitch = 256 * sizeof(XMFLOAT4);

	srand(time(NULL));

	XMFLOAT4* randomVectors = new XMFLOAT4[256 * 256];
	for (int i = 0; i < 256; ++i)
	{
		for (int j = 0; j < 256; ++j)
		{
			XMFLOAT4 random = XMFLOAT4(randf(0.0f, 1.0f), randf(0.0f, 1.0f), randf(0.0f, 1.0f), 0.0f);

			randomVectors[i * 256 + j] = random;
		}
	}

	initData.pSysMem = randomVectors;

	Texture* randomMap = new Texture(m_device, m_deviceContext, 256, 256, &initData);

	// Create the ambient occlusion texture
	DeferredBuffers* buffers = 0;
	buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetScreenWidth() / 2, m_Direct3D->GetScreenHeight() / 2, 1, 1);

	// Create the ambient occlusion shader
	BaseShader* shader = 0;
	shader = new AmbientOcclusionShader(m_device, m_hwnd);
	shader->AddTexture(m_resourceContainer->AddTexture(L"RANDOM_VECTOR_MAP", randomMap));
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"DEPTH_BUFFER"));
	shader->AddBuffers(m_resourceContainer->AddBuffer(L"AMBIENT_OCCLUSION_TEXTURE", buffers));
	shader->AddCamera(m_resourceContainer->GetCameraIndex(L"MAIN_CAMERA"));
	shader->AddObject(GetObjectByName(L"RENDER_WINDOW"));
	m_resourceContainer->AddShader(L"AMBIENT_OCCLUSION_SHADER", shader);

	// Create the bilateral blur buffer
	buffers = 0;
	buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetScreenWidth() / 2, m_Direct3D->GetScreenHeight() / 2, 1, 1);

	// Create the bilateral blur shader
	shader = 0;
	shader = new BilateralBlurShader(m_device, m_hwnd);
	shader->AddBuffers(m_resourceContainer->AddBuffer(L"BILATERAL_BLUR_BUFFER", buffers));
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"AMBIENT_OCCLUSION_TEXTURE"));
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"DEPTH_BUFFER"));
	shader->AddObject(GetObjectByName(L"RENDER_WINDOW"));
	m_resourceContainer->AddShader(L"BILATERAL_BLUR_SHADER", shader);
}

void BaseScene::InitDiskDoF(int sampleCount)
{
	// Create the output texture for the dof pass
	DeferredBuffers* buffers = new DeferredBuffers(m_device, m_deviceContext, m_Direct3D->GetScreenWidth(), m_Direct3D->GetScreenHeight(), 1, sampleCount);

	// Create the float parameters for the shader
	FloatParameter* focalWidth = new FloatParameter(1, NULL);
	FloatParameter* focalDistance = new FloatParameter(1, NULL);
	FloatParameter* farClip = new FloatParameter(1, NULL);
	FloatParameter* maxCoC = new FloatParameter(1, NULL);

	// Create the dof disk shader
	BaseShader* shader = new DoFDiskShader(m_device, m_hwnd);
	shader->AddBuffers(m_resourceContainer->AddBuffer(L"DOF_DISK_BUFFER", buffers));
	shader->AddBuffers(m_resourceContainer->GetBufferIndex(L"DEPTH_BUFFER"));
	shader->AddFloat(m_resourceContainer->AddFloat(L"DOF_FOCAL_WIDTH", focalWidth));
	shader->AddFloat(m_resourceContainer->AddFloat(L"DOF_FOCAL_DISTANCE", focalDistance));
	shader->AddFloat(m_resourceContainer->AddFloat(L"DOF_FAR_CLIP", farClip));
	shader->AddFloat(m_resourceContainer->AddFloat(L"DOF_MAX_COC", maxCoC));
	m_resourceContainer->AddShader(L"DOF_DISK_SHADER", shader);

	// Create the buffer copying shader for this effect
	shader = 0;
	shader = new BufferScalingShader(m_device, m_hwnd);
	reinterpret_cast<BufferScalingShader*>(shader)->SetBuffers(buffers->GetShaderResourceView(0), m_Direct3D->GetIntermediateBufferRenderTarget());
	m_resourceContainer->AddShader(L"DOF_BUFFER_SCALING_SHADER", shader);
}

void BaseScene::CaptureSceneBuffers()
{
	ID3D11Resource* resource;

	m_Direct3D->GetIntermediateBufferResourceView()->GetResource(&resource);
	wstring filepath = L"../res/data/buffers/INTERMEDIATE_BUFFER.dds";
	SaveDDSTextureToFile(m_deviceContext, resource, filepath.c_str());

	for (int i = 0; i < m_resourceContainer->GetBufferCount(); i++)
	{
		for (int k = 0; k < m_resourceContainer->GetBuffers(i)->GetBufferCount(); k++)
		{
			m_resourceContainer->GetBuffers(i)->GetShaderResourceView(k)->GetResource(&resource);
			filepath = L"../res/data/buffers/" + m_resourceContainer->GetBufferName(i)+ L".dds";
			SaveDDSTextureToFile(m_deviceContext, resource, filepath.c_str());
		}
	}
}