#ifndef _SCENE_H_
#define _SCENE_H_

#include "../DirectXTK/Inc/ScreenGrab.h"

#include "INIReader.h"
#include "ResourceContainer.h"
#include "SceneObject.h"
#include "Input.h"

#include "BufferScalingShader.h"
#include "PrePassGBufferShader.h"
#include "PrePassMaskShader.h"
#include "PrePassLightShader.h"
#include "PrePassFinalShader.h"
#include "DepthShader.h"
#include "AmbientOcclusionShader.h"
#include "BilateralBlurShader.h"
#include "ShadowMapShader.h"
#include "DoFDiskShader.h"
#include "FxaaShader.h"

class BaseScene
{
public:
	BaseScene(D3D* d3d, HWND hwnd, INIReader* iniReader);
	BaseScene(BaseScene& copy);
	~BaseScene();

	virtual void Update(Input* input, float frameTime) = 0;
	virtual void Render() = 0;

	SceneObject* GetObjectByName(const std::wstring& name);

protected:
	virtual void HandleInput(Input* input, float frameTime) = 0;
	void InitRendererNoAA();
	void InitRendererAA(int sampleCount);
	void InitFXAA();
	void InitDepthShader();
	void InitShadowMapShader();
	void InitAmbientOcclusion();
	void InitDiskDoF(int sampleCount);
	

	void CaptureSceneBuffers();

protected:
	ResourceContainer* m_resourceContainer;
	std::vector<SceneObject*> m_objects;

	D3D* m_Direct3D;
	ID3D11DeviceContext* m_deviceContext;
	ID3D11Device* m_device;
	HWND m_hwnd;

	DoFSetting m_dofSetting;
};
#endif