#include "BufferScalingShader.h"

BufferScalingShader::BufferScalingShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	m_deferredShader = true;
	InitShader(L"../res/shaders/buffer_scaling_vs.hlsl", L"../res/shaders/buffer_scaling_ps.hlsl");
}

BufferScalingShader::~BufferScalingShader()
{
	BaseShader::~BaseShader();
}

void BufferScalingShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
	loadVertexShader(vsFilename);
	loadPixelShader(psFilename);

	m_blendState = D3D::instance()->CreateDefaultBlendState();
	m_depthStencilState = D3D::instance()->CreateDefaultDepthStencilState();
	m_rasterizerState = D3D::instance()->CreateDefaultRasterizerState();

	m_resolutionBuffer = D3D::instance()->CreateConstantBuffer(sizeof(ResolutionDataBuffer), true, true, NULL);

	m_renderWindow = new QuadMesh(D3D::instance()->GetDevice(), D3D::instance()->GetDeviceContext(), NULL);
}

void BufferScalingShader::SetBuffers(ID3D11ShaderResourceView* src, ID3D11RenderTargetView* dest)
{
	m_src = src;
	m_dest = dest;
}

void BufferScalingShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	ID3D11Resource* bufferRes;
	ID3D11Texture2D* textureRes;
	D3D11_TEXTURE2D_DESC textureDesc;
	int srcX, srcY, destX, destY;
	ResolutionDataBuffer* resolutionDataBuffer;
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	ConfigurePipeline(deviceContext);

	// get the source dimensions
	m_src->GetResource(&bufferRes);
	textureRes = (ID3D11Texture2D*)bufferRes;
	textureRes->GetDesc(&textureDesc);
	srcX = textureDesc.Width;
	srcY = textureDesc.Height;

	// get the dest dimensions
	m_dest->GetResource(&bufferRes);
	textureRes = (ID3D11Texture2D*)bufferRes;
	textureRes->GetDesc(&textureDesc);
	destX = textureDesc.Width;
	destY = textureDesc.Height;

	// set the destination buffer as the output of the shader
	float color[] = { 1.0, 1.0f, 1.0f, 0.0f };
	deviceContext->OMSetRenderTargets(1, &m_dest, NULL);
	deviceContext->ClearRenderTargetView(m_dest, color);

	d3d->SetBackBufferViewport();

	// set the source buffer as the input of the shader
	deviceContext->PSSetShaderResources(0, 1, &m_src);

	// send the render window mesh to the shader
	m_renderWindow->SendData(deviceContext);
	m_indexCount = m_renderWindow->GetIndexCount();
	m_renderType = m_renderWindow->GetRenderType();

	// send the resolution data to the shader
	deviceContext->Map(m_resolutionBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
	resolutionDataBuffer = (ResolutionDataBuffer*)mappedResource.pData;
	
	resolutionDataBuffer->resolutionWidth = srcX;
	resolutionDataBuffer->resolutionHeight = srcY;
	resolutionDataBuffer->windowWidth = destX;
	resolutionDataBuffer->windowHeight = destY;
	deviceContext->Unmap(m_resolutionBuffer, 0);
	deviceContext->PSSetConstantBuffers(0, 1, &m_resolutionBuffer);

	// execute the shader
	BaseShader::Render(deviceContext, NULL, NULL);

	ClearPipeline(deviceContext);
}