#ifndef _D3D_H_
#define _D3D_H_

#include <Windows.h>
#include <d3d11.h>
#include <DirectXMath.h>
#include <stdlib.h>

#include "../DirectXTK/Inc/ScreenGrab.h"

using namespace DirectX;

enum class AntiAliasingSetting
{
	DISABLED,
	FXAA,
	MSAA2X,
	MSAA4X,
	MSAA8X,
	MSAA16X,
	TXAA
};

enum class DoFSetting
{
	DISABLED,
	GAUSSIAN,
	GAUSSIAN_DEPTH,
	BOKEH
};

class D3D
{
public:
	void* operator new(size_t i)
	{
		return _mm_malloc(i, 16);
	}

	void operator delete(void* p)
	{
		_mm_free(p);
	}

	static D3D *instance()
	{
		if (!m_Direct3DHandle)
			m_Direct3DHandle = new D3D();
		return m_Direct3DHandle;
	}
	void init(int windowWidth, int windowHeight, int resolutionWidth, int resolutionHeight, bool vsync, HWND hwnd, bool fullscreen, AntiAliasingSetting antiAliasingType);
	~D3D();

	void BeginScene(float r, float g, float b, float a);
	void EndScene();

	ID3D11Device* GetDevice();
	ID3D11DeviceContext* GetDeviceContext();
	ID3D11DepthStencilView* GetDepthStencilView();

	ID3D11Buffer* CreateVertexBuffer(UINT size, bool dynamic, bool streamout, D3D11_SUBRESOURCE_DATA* pData);
	ID3D11Buffer* CreateIndexBuffer(UINT size, bool dynamic, D3D11_SUBRESOURCE_DATA* pData);
	ID3D11Buffer* CreateConstantBuffer(UINT size, bool dynamic, bool CPUupdates, D3D11_SUBRESOURCE_DATA* pData);

	ID3D11DepthStencilState* CreateDefaultDepthStencilState();
	ID3D11BlendState* CreateDefaultBlendState();
	ID3D11RasterizerState* CreateDefaultRasterizerState();

	void GetVideoCardInfo(char*, int&);

	void SetIntermediateBufferRenderTarget();
	void SetBackBufferViewport();
	void ResetViewport();

	ID3D11RenderTargetView* GetIntermediateBufferRenderTarget();
	ID3D11ShaderResourceView* GetIntermediateBufferResourceView();

	int GetScreenWidth();
	int GetScreenHeight();

	int GetResolutionWidth();
	int GetResolutionHeight();

	AntiAliasingSetting GetAntiAliasingSetting();

protected:

protected:
	bool m_vsync_enabled;
	int m_videoCardMemory;
	int m_sampleCount;
	char m_videoCardDescription[128];
	IDXGISwapChain* m_swapChain;
	ID3D11Device* m_device;
	ID3D11DeviceContext* m_deviceContext;
	
	// Back buffer
	ID3D11RenderTargetView* m_backBufferTargetView;
	ID3D11Texture2D* m_backBufferDepthStencilBuffer;
	ID3D11DepthStencilView* m_backBufferDepthStencilView;
	D3D11_VIEWPORT m_backBufferViewport;

	// Intermediate buffers
	ID3D11RenderTargetView* m_intermediateBufferTargetView;
	ID3D11Texture2D* m_intermediateDepthStencilBuffer;
	ID3D11DepthStencilView* m_intermediateBufferDepthStencilView;
	ID3D11ShaderResourceView* m_intermediateBufferShaderResourceView;

	ID3D11RenderTargetView* m_intermediateBufferTargetViewB;
	ID3D11Texture2D* m_intermediateDepthStencilBufferB;
	ID3D11DepthStencilView* m_intermediateBufferDepthStencilViewB;
	ID3D11ShaderResourceView* m_intermediateBufferShaderResourceViewB;

	D3D11_VIEWPORT m_intermediateBufferViewport;


	int m_windowWidth, m_windowHeight;
	int m_resolutionWidth, m_resolutionHeight;
	AntiAliasingSetting m_aaSetting;

	static D3D* m_Direct3DHandle;

	void* m_bufferScalingShader;

};


#endif