#include "DoFDiskShader.h"
#include "ResourceContainer.h"

DoFDiskShader::DoFDiskShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	m_postProcessShader = true;
	InitShader(L"../res/shaders/post_process_vs.hlsl", L"../res/shaders/dof_disk_blur_ps.hlsl");
}

DoFDiskShader::~DoFDiskShader()
{
	if (m_blurFactorsBuffer)
	{
		m_blurFactorsBuffer->Release();
		m_blurFactorsBuffer = 0;
	}

	BaseShader::~BaseShader();
}

void DoFDiskShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
	loadVertexShader(vsFilename);
	loadPixelShader(psFilename);

	m_blurFactorsBuffer = D3D::instance()->CreateConstantBuffer(sizeof(PSBlurFactorsBuffer), true, true, NULL);

	m_renderWindow = new QuadMesh(D3D::instance()->GetDevice(), D3D::instance()->GetDeviceContext(), NULL);
}

void DoFDiskShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	float width, height;
	D3D11_TEXTURE2D_DESC texDesc;
	ID3D11Resource* bufferRes;
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	PSBlurFactorsBuffer* blurFactorsBuffer;

	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	ConfigurePipeline(deviceContext);

	container->GetBuffers(m_renderBuffers[0])->SetRenderTargets(NULL);
	container->GetBuffers(m_renderBuffers[0])->ClearRenderTargets(0.0f, 0.0f, 0.0f, 0.0f);

	// set the source and depth textures as shader input
	ID3D11ShaderResourceView* srcTexture = D3D::instance()->GetIntermediateBufferResourceView();
	deviceContext->PSSetShaderResources(0, 1, &srcTexture);
	srcTexture->GetResource(&bufferRes);
	ID3D11Texture2D* tex2D = (ID3D11Texture2D*)bufferRes;
	tex2D->GetDesc(&texDesc);
	width = 0.5f / (float)texDesc.Width;
	height = 0.5f / (float)texDesc.Height;

	srcTexture = container->GetBuffers(m_renderBuffers[1])->GetShaderResourceView(0);
	deviceContext->PSSetShaderResources(1, 1, &srcTexture);

	// set up the DOF taps
	XMFLOAT2 discOffsets[] =
	{
		XMFLOAT2(-0.326212f * width, -0.40581f * height),
		XMFLOAT2(-0.840144f * width, -0.07358f * height),
		XMFLOAT2(-0.840144f * width, 0.457137f * height),
		XMFLOAT2(-0.203345f * width, 0.620716f * height),
		XMFLOAT2(0.96234f * width, -0.194983f * height),
		XMFLOAT2(0.473434f * width, -0.480026f * height),
		XMFLOAT2(0.519456f * width, 0.767022f * height),
		XMFLOAT2(0.185461f * width, -0.893124f * height),
		XMFLOAT2(0.507431f * width, 0.064425f * height),
		XMFLOAT2(0.89642f * width, 0.412458f * height),
		XMFLOAT2(-0.32194f * width, -0.932615f * height),
		XMFLOAT2(-0.791559f * width, -0.59771f * height)
	};

	// send the blur factors to the shader
	deviceContext->Map(m_blurFactorsBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
	blurFactorsBuffer = (PSBlurFactorsBuffer*)mappedResource.pData;
	blurFactorsBuffer->focalWidth = container->GetFloat(m_renderFloats[0])->GetParameter(0);
	blurFactorsBuffer->focalDistance = container->GetFloat(m_renderFloats[1])->GetParameter(0);
	blurFactorsBuffer->farClip= container->GetFloat(m_renderFloats[2])->GetParameter(0);
	blurFactorsBuffer->maxCoC = container->GetFloat(m_renderFloats[3])->GetParameter(0);
	//memcpy(blurFactorsBuffer->filterTaps, discOffsets, sizeof(XMFLOAT2) * 12);
	deviceContext->Unmap(m_blurFactorsBuffer, 0);
	deviceContext->PSSetConstantBuffers(0, 1, &m_blurFactorsBuffer);

	// send the render window mesh to the shader
	m_renderWindow->SendData(deviceContext);
	m_indexCount = m_renderWindow->GetIndexCount();
	m_renderType = m_renderWindow->GetRenderType();

	// execute the shader
	BaseShader::Render(deviceContext, NULL, NULL);

	ClearPipeline(deviceContext);
}