#include "FxaaShader.h"
#include "ResourceContainer.h"

FxaaShader::FxaaShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	m_postProcessShader = true;
	InitShader(L"../res/shaders/post_process_vs.hlsl", L"../res/shaders/fxaa_ps.hlsl");
}

FxaaShader::~FxaaShader()
{
	if (m_fxaaFactorsBuffer)
	{
		m_fxaaFactorsBuffer->Release();
		m_fxaaFactorsBuffer = 0;
	}
}

void FxaaShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
	D3D11_SAMPLER_DESC samplerDesc;

	loadVertexShader(vsFilename);
	loadPixelShader(psFilename);

	// Create a texture sampler state description.
	samplerDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
	samplerDesc.AddressU = D3D11_TEXTURE_ADDRESS_CLAMP;
	samplerDesc.AddressV = D3D11_TEXTURE_ADDRESS_CLAMP;
	samplerDesc.AddressW = D3D11_TEXTURE_ADDRESS_CLAMP;
	samplerDesc.MipLODBias = 0.0f;
	samplerDesc.MaxAnisotropy = 16;
	samplerDesc.ComparisonFunc = D3D11_COMPARISON_ALWAYS;
	samplerDesc.BorderColor[0] = 0;
	samplerDesc.BorderColor[1] = 0;
	samplerDesc.BorderColor[2] = 0;
	samplerDesc.BorderColor[3] = 0;
	samplerDesc.MinLOD = 0;
	samplerDesc.MaxLOD = D3D11_FLOAT32_MAX;

	// Create the texture sampler state.
	m_device->CreateSamplerState(&samplerDesc, &m_sampleState);

	// Create the fxaa factors constant buffer
	m_fxaaFactorsBuffer = D3D::instance()->CreateConstantBuffer(sizeof(FxaaFactorsBuffer), true, true, NULL);
}

void FxaaShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	FxaaFactorsBuffer* fxaaFactorsBuffer;

	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	ConfigurePipeline(deviceContext);

	container->GetBuffers(m_renderBuffers[0])->SetRenderTargets(NULL);
	container->GetBuffers(m_renderBuffers[0])->ClearRenderTargets(0.0f, 0.0f, 0.0f, 0.0f);

	// set the source texture as shader input
	ID3D11ShaderResourceView* srcTexture = D3D::instance()->GetIntermediateBufferResourceView();
	deviceContext->PSSetShaderResources(0, 1, &srcTexture);

	// set the sampler state in the shader
	deviceContext->PSSetSamplers(0, 1, &m_sampleState);

	// send the fxaa factors data to the shader
	deviceContext->Map(m_fxaaFactorsBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
	fxaaFactorsBuffer = (FxaaFactorsBuffer*)mappedResource.pData;
	fxaaFactorsBuffer->subpixelAliasingRemoval = 0.75f;
	fxaaFactorsBuffer->edgeThreshold = 0.166f;
	fxaaFactorsBuffer->edgeThresholdMin = 0.0833f;
	fxaaFactorsBuffer->gather4Alpha = 1.0f;
	fxaaFactorsBuffer->discard = 0.0f;
	fxaaFactorsBuffer->greenAsLuma = 0.0f;
	fxaaFactorsBuffer->qualityRcpFrame = XMFLOAT2(1.0f / d3d->GetResolutionWidth(), 1.0f / d3d->GetResolutionHeight());
	fxaaFactorsBuffer->qualityPreset = 5.0f;
	fxaaFactorsBuffer->padding = XMFLOAT3(0.0f, 0.0f, 0.0f);
	deviceContext->Unmap(m_fxaaFactorsBuffer, 0);
	deviceContext->PSSetConstantBuffers(0, 1, &m_fxaaFactorsBuffer);

	// send the render window mesh data to the shader
	container->GetMesh(L"RENDER_WINDOW")->SendData(deviceContext);
	m_indexCount = container->GetMesh(L"RENDER_WINDOW")->GetIndexCount();
	m_renderType = container->GetMesh(L"RENDER_WINDOW")->GetRenderType();

	// execute the shader
	BaseShader::Render(deviceContext, NULL, NULL);

	ClearPipeline(deviceContext);
}