#ifndef _LIGHT_H_
#define _LIGHT_H_

#include <directxmath.h>

#include "DeferredBuffers.h"

using namespace DirectX;

struct LightBufferType
{
	XMFLOAT4 lightPos;
	XMFLOAT4 lightColor;
	XMFLOAT4 lightDirection;
	XMFLOAT4 lightRange;
	XMFLOAT2 spotlightAngles;
	float lightType;
	float padding;
};

class Light
{

public:
	void* operator new(size_t i)
	{
		return _mm_malloc(i, 16);
	}

	void operator delete(void* p)
	{
		_mm_free(p);
	}

	~Light();

	void GenerateViewMatrix();
	void GenerateProjectionMatrix(float, float);

	// Setters
	void SetDiffuseColour(float, float, float, float);
	void SetDirection(float, float, float);
	void SetPosition(float, float, float);
	void SetLookAt(float, float, float);
	void SetConstantAttenuation(float);
	void SetLinearAttenuation(float);
	void SetQuadraticAttenuation(float);
	void SetLightType(int);

	// Getters
	XMFLOAT4 GetDiffuseColour();
	XMFLOAT3 GetDirection();
	XMFLOAT3 GetPosition();
	XMMATRIX GetViewMatrix();
	XMMATRIX GetProjectionMatrix();
	float GetConstantAttenuation();
	float GetLinearAttenuation();
	float GetQuadraticAttenuation();
	float GetLightType();

	void GenerateShadowMapBuffer(ID3D11Device* device, ID3D11DeviceContext* deviceContext, int width, int height, int mapCount, int sampleCount);
	DeferredBuffers* GetShadowMap();

protected:
	DeferredBuffers* m_shadowMap;
	XMFLOAT4 m_diffuseColour;
	XMFLOAT3 m_direction;
	XMVECTOR m_position;
	XMMATRIX m_viewMatrix;
	XMMATRIX m_projectionMatrix;
	XMVECTOR m_lookAt;
	float m_constantAttenuation;
	float m_linearAttenuation;
	float m_quadraticAttenuation;
	float m_lightType;
};

#endif