#ifndef _SCENEOBJECT_H_
#define _SCENEOBJECT_H_

#include "BaseMesh.h"

class SceneObject
{
public:
	SceneObject();
	SceneObject(SceneObject& copy);
	~SceneObject();

	void AddMesh(int meshIndex);
	void RemoveMesh(int mesh);
	void RemoveMeshes(int meshIndex);

	void AddTexture(int textureIndex);
	void RemoveTexture(int texture);
	void RemoveTextures(int textureIndex);

	void AttachToShader(int shaderIndex);
	void DetachFromShader(int shaderIndex);

	std::vector<int>& GetMeshes();

	int GetMeshIndex(int mesh);
	int GetTextureIndex(int texture);

	void SetPosition(XMFLOAT3 position);
	void SetRotation(XMFLOAT3 rotation);
	void SetScale(XMFLOAT3 scale);

	XMFLOAT3 GetPosition();
	void GetPosition(float& x, float& y, float& z);
	
	XMFLOAT3 GetRotation();
	void GetRotation(float& x, float& y, float& z);

	XMFLOAT3 GetScale();
	void GetScale(float& x, float& y, float& z);

	XMMATRIX GetTransformMatrix();

	void SetName(const std::wstring& name);
	std::wstring GetName();

	void SetSpecularAlbedo(float r, float g, float b);
	XMFLOAT3 GetSpecularAlbedo();

	void SetSpecularPower(float pow);
	float GetSpecularPower();

protected:
	void RecalculateTransformMatrix();

protected:
	std::vector<int> m_renderShaders;
	std::vector<int> m_meshes;
	std::vector<int> m_textures;

	XMFLOAT3 m_position;
	XMFLOAT3 m_rotation;
	XMFLOAT3 m_scale;
	XMMATRIX m_transformMatrix;
	bool m_transformUpdate;

	XMFLOAT3 m_specularAlbedo;
	float m_specularPower;

	std::wstring m_objectName;

	bool m_visible;
};
#endif