#include "../res/shaders/shader_settings.hlsli"

#if SAMPLE_COUNT > 1
Texture2DMS<float4, SAMPLE_COUNT> srcBuffer : register(t0);
#else
Texture2D srcBuffer : register(t0);
#endif

cbuffer ResolutionData : register(b0)
{
	float resolutionWidth;
	float resolutionHeight;
	float windowWidth;
	float windowHeight;
}

#if SAMPLE_COUNT > 1
float4 main(in float4 screenPos : SV_POSITION) : SV_Target
{
	// set the sample indices for the screen
	int3 sampleIndices = int3(screenPos.xy, 0);

	float widthRatio = resolutionWidth / windowWidth;
	float heightRatio = resolutionHeight / windowHeight;

	sampleIndices.x = sampleIndices.x * widthRatio;
	sampleIndices.y = sampleIndices.y * heightRatio;

	float4 bufferColor = float4(0.0f, 0.0f, 0.0f, 0.0f);

	for (int i = 0; i < SAMPLE_COUNT; ++i)
	{
		float4 sampleColor = srcBuffer.Load(sampleIndices.xy, i);
		bufferColor += sampleColor;
	}
	
	bufferColor = bufferColor / (float)SAMPLE_COUNT;

	return bufferColor;
}
#else
float4 main(in float4 screenPos : SV_POSITION) : SV_Target
{
	// set the sample indices for the screen
	int3 sampleIndices = int3(screenPos.xy, 0);

	float widthRatio = resolutionWidth / windowWidth;
	float heightRatio = resolutionHeight / windowHeight;

	sampleIndices.x = sampleIndices.x * widthRatio;
	sampleIndices.y = sampleIndices.y * heightRatio;

	float4 bufferColor = float4(0.0f, 0.0f, 0.0f, 0.0f);

	bufferColor = srcBuffer.Load(sampleIndices);

	return bufferColor;
}
#endif