Texture2D normalMap : register(t0);
SamplerState bumpSampler : register(s0);

// Constants
cbuffer MatrixBuffer
{
	matrix inverseView;
};

//I/O Structs
struct PS_INPUT
{
	float4 position : SV_POSITION;
	float2 texCoord : TEXCOORD0;
	float3 viewPosition : POSITION;
	float3 worldNormal : NORMAL;
	float3 worldTangent : TANGENT;
	float3 worldBinormal : BINORMAL;
};

float4 main(PS_INPUT input) : SV_TARGET
{
	//normalize the tangent frame after interpolation
	float3x3 tangentFrame = float3x3(normalize(input.worldTangent), normalize(input.worldBinormal), normalize(input.worldNormal));

	//Sample the tangent space normal map and decompress
	float3 normalTS = normalMap.Sample(bumpSampler, input.texCoord).rgb;
	normalTS = normalize(normalTS * 2.0f - 1.0f);

	//convert to world space
	float3 worldNormal = mul(normalTS, tangentFrame);

	// convert the normals to view
	float3 viewNormal = mul(worldNormal, (float3x3)inverseView);

	return float4(viewNormal, input.viewPosition.z);
}