//constant buffers
cbuffer MatrixParameters
{
	matrix worldMatrix;
	matrix viewMatrix;
	matrix projectionMatrix;
};

//I/O Structs
struct VS_INPUT
{
	float3 position : POSITION;
	float2 texCoord : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
};

struct VS_OUTPUT
{
	float4 position : SV_POSITION;
	float2 texCoord : TEXCOORD0;
	float3 viewPosition : POSITION;
	float3 worldNormal : NORMAL;	
	float3 worldTangent : TANGENT;
	float3 worldBinormal : BINORMAL;
};

VS_OUTPUT main( VS_INPUT input )
{
	VS_OUTPUT output;

	output.position = mul(float4(input.position, 1.0f), worldMatrix);
	output.position = mul(output.position, viewMatrix);
	output.viewPosition = output.position;
	output.position = mul(output.position, projectionMatrix);

	float3 worldNormal = mul(input.normal, worldMatrix);
	
	//reconstruct the rest of the tangent frame
	float3 worldTangent = normalize(mul(input.tangent.xyz, (float3x3)worldMatrix));
	float3 worldBinormal = normalize(cross(worldNormal, worldTangent)) * input.tangent.w;

	output.worldNormal = worldNormal;
	output.worldTangent = worldTangent;
	output.worldBinormal = worldBinormal;

	output.texCoord = input.texCoord;

	return output;
}