// textures
Texture2D sourceTexture : register(t0);
Texture2D blurTexture : register(t1);
Texture2D depthTexture : register(t2);

// I/O Stream
cbuffer BlurFactors : register(b0)
{
	float attenuation;
	float focalWidth;
	float focalDistance;
	float farClip;
}

float GetBlurFactor(float depth)
{
	return smoothstep(0, focalWidth, abs(focalDistance - (depth * farClip)));
}

float4 main(float4 position : SV_Position) : SV_TARGET
{
	int3 sampleIndices = int3(position.xy, 0);
	float4 originalColor = sourceTexture.Load(sampleIndices.xy, 0);
	float4 blurredColor = blurTexture.Load(sampleIndices.xy, 0);
	float depth = depthTexture.Load(sampleIndices.xy, 0).x;

	float blurFactor = GetBlurFactor(depth);

	return lerp(originalColor, blurredColor, saturate(blurFactor) * attenuation);
}