// Source textures
Texture2D sourceTexture : register(t0);

// Samplers
SamplerState sourceSampler : register(s0);

// I/O Structs
struct VertexOutput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
};

// Constant
cbuffer FxaaFactors : register(b0)
{
	float fxaaSubpixelAliasingRemoval;
	float fxaaEdgeThreshold;
	float fxaaEdgeThresholdMin;
	float fxaaGather4Alpha;
	float fxaaDiscard;
	float fxaaGreenAsLuma;
	float2 fxaaQualityRcpFrame;
	float fxaaQualityPreset;
	float3 padding;
}

struct FxaaQualityLevel
{
	float QUALITY_PS;
	float QUALITY_P0;
	float QUALITY_P1;
	float QUALITY_P2;
	float QUALITY_P3;
	float QUALITY_P4;
	float QUALITY_P5;
	float QUALITY_P6;
	float QUALITY_P7;
	float QUALITY_P8;
	float QUALITY_P9;
	float QUALITY_P10;
	float QUALITY_P11;
};

/*FxaaQualityLevel FxaaQualityLevels[17] =
{
	{ 3.0f, 1.5f, 3.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 4.0f, 1.0f, 1.5f, 3.0f, 12.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 5.0f, 1.0f, 1.5f, 2.0f, 4.0f, 12.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 6.0f, 1.0f, 1.5f, 2.0f, 2.0f, 4.0f, 12.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 7.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 4.0f, 12.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 8.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 2.0f, 4.0f, 12.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 3.0f, 1.5f, 2.0f, 8.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 4.0f, 1.0f, 1.5f, 2.0f, 8.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 5.0f, 1.0f, 1.5f, 2.0f, 2.0f, 8.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 6.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 8.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 7.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 3.0f, 8.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 8.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 2.0f, 4.0f, 8.0f, 0.0f, 0.0f, 0.0f, 0.0f },
	{ 9.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 4.0f, 8.0f, 0.0f, 0.0f, 0.0f },
	{ 10.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 4.0f, 8.0f, 0.0f, 0.0f },
	{ 11.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 4.0f, 8.0f, 0.0f },
	{ 12.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 2.0f, 4.0f, 8.0f },
	{ 12.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.5f, 2.0f, 2.0f, 2.0f, 2.0f, 4.0f, 8.0f },
};*/

float2 ScreenPosToTexCoord(float2 screenPos)
{
	float2 texCoord;
	float xSize = 1.0f / fxaaQualityRcpFrame.x;
	float ySize = 1.0f / fxaaQualityRcpFrame.y;
	texCoord.x = screenPos.x / xSize;
	texCoord.y = screenPos.y / ySize;
	return texCoord;
}

float4 FxaaTexTop(Texture2D tex, SamplerState smpl, float2 pos)
{
	return tex.SampleLevel(smpl, pos, 0.0);
}

float4 FxaaTexOff(Texture2D tex, SamplerState smpl, float2 pos, int2 offset)
{
	return tex.SampleLevel(smpl, pos, 0.0, offset);
}

float4 FxaaTexAlpha4(Texture2D tex, SamplerState smpl, float2 pos)
{
	return tex.GatherAlpha(smpl, pos);
}

float4 FxaaTexOffAlpha4(Texture2D tex, SamplerState smpl, float2 pos, int2 offset)
{
	return tex.GatherAlpha(smpl, pos, offset);
}

float4 FxaaTexGreen4(Texture2D tex, SamplerState smpl, float2 pos)
{
	return tex.GatherGreen(smpl, pos);
}

float4 FxaaTexOffGreen4(Texture2D tex, SamplerState smpl, float2 pos, int2 offset)
{
	return tex.GatherGreen(smpl, pos, offset);
}

float FxaaLuma(in float4 rgba)
{
	if (fxaaGreenAsLuma == 0)
	{
		return rgba.w;
	}
	else
	{
		return rgba.y;
	}
}

float4 main(VertexOutput input) : SV_TARGET
{
	float4 luma4A, luma4B, rgbyM;
	float2 posM = input.tex;
	float lumaM, lumaE, lumaS, lumaSE, lumaNW, lumaNE, lumaSW, lumaN, lumaW;
	lumaM = 0.0f;

	FxaaQualityLevel FxaaQualityLevels[17];

	FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS = 8;
	FxaaQualityLevels[fxaaQualityPreset].QUALITY_P0 = 1.0f;
	FxaaQualityLevels[fxaaQualityPreset].QUALITY_P1 = 1.5f;
	FxaaQualityLevels[fxaaQualityPreset].QUALITY_P2 = 2.0f;
	FxaaQualityLevels[fxaaQualityPreset].QUALITY_P3 = 2.0f;
	FxaaQualityLevels[fxaaQualityPreset].QUALITY_P4 = 2.0f;
	FxaaQualityLevels[fxaaQualityPreset].QUALITY_P5 = 2.0f;
	FxaaQualityLevels[fxaaQualityPreset].QUALITY_P6 = 4.0f;
	FxaaQualityLevels[fxaaQualityPreset].QUALITY_P7 = 12.0f;

	if (fxaaGather4Alpha == 1)
	{
		if (fxaaDiscard == 0)
		{
			rgbyM = FxaaTexTop(sourceTexture, sourceSampler, posM);
			if (fxaaGreenAsLuma == 0)
			{
				lumaM = rgbyM.w;
			}
			else
			{
				lumaM = rgbyM.y;
			}
		}
		
		if (fxaaGreenAsLuma == 0)
		{
			luma4A = FxaaTexAlpha4(sourceTexture, sourceSampler, posM);
			luma4B = FxaaTexOffAlpha4(sourceTexture, sourceSampler, posM, int2(-1, -1));
		}
		else
		{
			luma4A = FxaaTexGreen4(sourceTexture, sourceSampler, posM);
			luma4B = FxaaTexOffGreen4(sourceTexture, sourceSampler, posM, int2(-1, -1));
		}

		if (fxaaDiscard == 1)
		{
			lumaM = luma4A.w;
		}
		lumaE = luma4A.z;
		lumaS = luma4A.x;
		lumaSE = luma4A.y;
		lumaNW = luma4B.w;
		lumaN = luma4B.z;
		lumaW = luma4B.x;
	}
	else
	{
		rgbyM = FxaaTexTop(sourceTexture, sourceSampler, posM);
		if (fxaaGreenAsLuma == 0)
		{
			lumaM = rgbyM.w;
		}
		else
		{
			lumaM = rgbyM.y;
		}

		lumaS = FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(0, 1)));
		lumaE - FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(1, 0)));
		lumaN = FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(0, -1)));
		lumaW = FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(-1, 0)));
	}

	float maxSM = max(lumaS, lumaM);
	float minSM = min(lumaS, lumaM);
	float maxESM = max(lumaE, maxSM);
	float minESM = min(lumaE, minSM);
	float maxWN = max(lumaN, lumaW);
	float minWN = min(lumaN, lumaW);
	float rangeMax = max(maxWN, maxESM);
	float rangeMin = min(minWN, minESM);
	float rangeMaxScaled = rangeMax * fxaaEdgeThreshold;
	float range = rangeMax - rangeMin;
	float rangeMaxClamped = max(fxaaEdgeThresholdMin, rangeMaxScaled);
	bool earlyExit = range < rangeMaxClamped;

	if (earlyExit)
	{
		if (fxaaDiscard == 1)
		{
			clip(-1.0f);
		}
		else
		{
			return rgbyM;
		}
	}

	if (fxaaGather4Alpha == 0)
	{
		lumaNW - FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(-1, -1)));
		lumaSE = FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(1, 1)));
		lumaNE = FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(1, -1)));
		lumaSW = FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(-1, 1)));
	}
	else
	{
		lumaNE = FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(1, -1)));
		lumaSW = FxaaLuma(FxaaTexOff(sourceTexture, sourceSampler, posM, int2(-1, 1)));
	}

	float lumaNS = lumaN + lumaS;
	float lumaWE = lumaW + lumaE;
	float subpixRange = 1.0f / range;
	float subpixNSWE = lumaNS + lumaWE;
	float edgeHorz1 = (-2.0f * lumaM) + lumaNS;
	float edgeVert1 = (-2.0f * lumaM) + lumaWE;

	float lumaNESE = lumaNE + lumaSE;
	float lumaNWNE = lumaNW + lumaNE;
	float edgeHorz2 = (-2.0f * lumaE) + lumaNESE;
	float edgeVert2 = (-2.0f * lumaN) + lumaNWNE;

	float lumaNWSW = lumaNW + lumaSW;
	float lumaSWSE = lumaSW + lumaSE;
	float edgeHorz3 = (-2.0f * lumaW) + lumaNWSW;
	float edgeVert3 = (-2.0f * lumaS) + lumaSWSE;
	float edgeHorz4 = (abs(edgeHorz1) * 2.0f) + abs(edgeHorz2);
	float edgeVert4 = (abs(edgeVert1) * 2.0f) + abs(edgeVert2);
	float edgeHorz = abs(edgeHorz3) + edgeHorz4;
	float edgeVert = abs(edgeVert3) + edgeVert4;

	float subpixNWSWNESE = lumaNWSW + lumaNESE;
	float lengthSign = fxaaQualityRcpFrame.x;
	bool horzSpan = edgeHorz >= edgeVert;
	float subpixA = subpixNSWE * 2.0f + subpixNWSWNESE;

	if (!horzSpan) lumaN = lumaW;
	if (!horzSpan) lumaS = lumaE;
	if (horzSpan) lengthSign = fxaaQualityRcpFrame.y;
	float subpixB = (subpixA * (1.0f / 12.0f)) - lumaM;

	float gradientN = lumaN - lumaM;
	float gradientS = lumaS - lumaM;
	float lumaNN = lumaN + lumaM;
	float lumaSS = lumaS + lumaM;
	bool pairN = abs(gradientN) >= abs(gradientS);
	float gradient = max(abs(gradientN), abs(gradientS));
	if (pairN) lengthSign = -lengthSign;
	float subpixC = saturate(abs(subpixB) * subpixRange);

	float2 posB = posM;
	float2 offNP;
	offNP.x = (!horzSpan) ? 0.0f : fxaaQualityRcpFrame.x;
	offNP.y = ( horzSpan) ? 0.0f : fxaaQualityRcpFrame.y;
	if (!horzSpan) posB.x += lengthSign * 0.5f;
	if ( horzSpan) posB.y += lengthSign * 0.5f;

	float2 posN;
	posN.x = posB.x - offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P0;
	posN.y = posB.y - offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P0;
	float2 posP;
	posP.x = posB.x + offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P0;
	posP.y = posB.y + offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P0;
	float subpixD = ((-2.0f) * subpixC) + 3.0f;
	float lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
	float subpixE = subpixC * subpixC;
	float lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));

	if (!pairN) lumaNN = lumaSS;
	float gradientScaled = gradient * 1.0f / 4.0f;
	float lumaMM = lumaM - lumaNN * 0.5f;
	float subpixF = subpixD * subpixE;
	bool lumaMLTZero = lumaMM < 0.0f;

	lumaEndN -= lumaNN * 0.5f;
	lumaEndP -= lumaNN * 0.5f;
	bool doneN = abs(lumaEndN) >= gradientScaled;
	bool doneP = abs(lumaEndP) >= gradientScaled;
	if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P1;
	if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P1;
	bool doneNP = (!doneN) || (!doneP);
	if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P1;
	if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P1;

	if (doneNP)
	{
		if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
		if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
		if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
		if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
		doneN = abs(lumaEndN) >= gradientScaled;
		doneP = abs(lumaEndP) >= gradientScaled;
		if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P2;
		if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P2;
		doneNP = (!doneN) || (!doneP);
		if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P2;
		if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P2;

		if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 3)
		{
			if (doneNP)
			{
				if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
				if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
				if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
				if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
				doneN = abs(lumaEndN) >= gradientScaled;
				doneP = abs(lumaEndP) >= gradientScaled;
				if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P3;
				if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P3;
				doneNP = (!doneN) || (!doneP);
				if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P3;
				if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P3;

				if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 4)
				{
					if (doneNP)
					{
						if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
						if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
						if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
						if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
						doneN = abs(lumaEndN) >= gradientScaled;
						doneP = abs(lumaEndP) >= gradientScaled;
						if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P4;
						if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P4;
						doneNP = (!doneN) || (!doneP);
						if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P4;
						if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P4;

						if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 5)
						{
							if (doneNP)
							{
								if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
								if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
								if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
								if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
								doneN = abs(lumaEndN) >= gradientScaled;
								doneP = abs(lumaEndP) >= gradientScaled;
								if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P5;
								if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P5;
								doneNP = (!doneN) || (!doneP);
								if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P5;
								if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P5;

								if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 6)
								{
									if (doneNP)
									{
										if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
										if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
										if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
										if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
										doneN = abs(lumaEndN) >= gradientScaled;
										doneP = abs(lumaEndP) >= gradientScaled;
										if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P6;
										if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P6;
										doneNP = (!doneN) || (!doneP);
										if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P6;
										if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P6;

										if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 7)
										{
											if (doneNP)
											{
												if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
												if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
												if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
												if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
												doneN = abs(lumaEndN) >= gradientScaled;
												doneP = abs(lumaEndP) >= gradientScaled;
												if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P7;
												if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P7;
												doneNP = (!doneN) || (!doneP);
												if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P7;
												if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P7;

												if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 8)
												{
													if (doneNP)
													{
														if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
														if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
														if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
														if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
														doneN = abs(lumaEndN) >= gradientScaled;
														doneP = abs(lumaEndP) >= gradientScaled;
														if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P8;
														if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P8;
														doneNP = (!doneN) || (!doneP);
														if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P8;
														if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P8;

														if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 9)
														{
															if (doneNP)
															{
																if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
																if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
																if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
																if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
																doneN = abs(lumaEndN) >= gradientScaled;
																doneP = abs(lumaEndP) >= gradientScaled;
																if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P9;
																if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P9;
																doneNP = (!doneN) || (!doneP);
																if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P9;
																if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P9;

																if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 10)
																{
																	if (doneNP)
																	{
																		if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
																		if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
																		if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
																		if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
																		doneN = abs(lumaEndN) >= gradientScaled;
																		doneP = abs(lumaEndP) >= gradientScaled;
																		if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P10;
																		if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P10;
																		doneNP = (!doneN) || (!doneP);
																		if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P10;
																		if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P10;

																		if (FxaaQualityLevels[fxaaQualityPreset].QUALITY_PS > 11)
																		{
																			if (doneNP)
																			{
																				if (!doneN) lumaEndN = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posN));
																				if (!doneP) lumaEndP = FxaaLuma(FxaaTexTop(sourceTexture, sourceSampler, posP));
																				if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5f;
																				if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5f;
																				doneN = abs(lumaEndN) >= gradientScaled;
																				doneP = abs(lumaEndP) >= gradientScaled;
																				if (!doneN) posN.x -= offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P11;
																				if (!doneN) posN.y -= offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P11;
																				doneNP = (!doneN) || (!doneP);
																				if (!doneP) posP.x += offNP.x * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P11;
																				if (!doneP) posP.y += offNP.y * FxaaQualityLevels[fxaaQualityPreset].QUALITY_P11;
																			}
																		}
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}

	float distN = posM.x - posN.x;
	float distP = posP.x - posM.x;
	if (!horzSpan) distN = posM.y - posN.y;
	if (!horzSpan) distP = posP.y - posM.y;

	bool goodSpanN = (lumaEndN < 0.0f) != lumaMLTZero;
	float spanLength = distP + distN;
	bool goodSpanP = (lumaEndP < 0.0f) != lumaMLTZero;
	float spanLengthRcp = 1.0f / spanLength;

	bool directionN = distN < distP;
	float dist = min(distN, distP);
	bool goodSpan = directionN ? goodSpanN : goodSpanP;
	float subpixG = subpixF * subpixF;
	float pixelOffset = (dist * (-spanLengthRcp)) + 0.5f;
	float subpixH = subpixG * fxaaSubpixelAliasingRemoval;

	float pixelOffsetGood = goodSpan ? pixelOffset : 0.0f;
	float pixelOffsetSubpix = max(pixelOffsetGood, subpixH);
	if (!horzSpan) posM.x += pixelOffsetSubpix * lengthSign;
	if ( horzSpan) posM.y += pixelOffsetSubpix * lengthSign;

	if (fxaaDiscard == 1)
	{
		return FxaaTexTop(sourceTexture, sourceSampler, posM);
	}
	else
	{
		return float4(FxaaTexTop(sourceTexture, sourceSampler, posM).xyz, lumaM);
	}
}