// textures
Texture2D sourceTexture : register(t0);

// I/O Structs
struct VertexOutput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
};

// buffers
cbuffer BlurFactors : register(b0)
{
	float blurRadius;
	float blurDirection;
	float2 sourceDimensions;
}

const float blurSigma = 0.5f;

float CalculateGaussianWeight(int samplePoint)
{
	float g = 1.0f / sqrt(2.0f * 3.14159 * blurSigma * blurSigma);
	return (g * exp(-(samplePoint * samplePoint) / (2.0f * blurSigma * blurSigma)));
}

float4 main(VertexOutput input) : SV_TARGET
{
	float4 position = input.pos;
	float4 color = float4(0.0f, 0.0f, 0.0f, 0.0f);
	int2 texCoord = position.xy;

	for (int i = -blurRadius; i < blurRadius; i++)
	{
		float weight = CalculateGaussianWeight(i);

		if (blurDirection == 0.0f)
			texCoord.x = position.x + (i / sourceDimensions.x);
		else
			texCoord.y = position.y + (i / sourceDimensions.y);

		float4 currentSample = sourceTexture.Load(texCoord, 0);
		color += currentSample * weight;
	}
	return color;
}