// Constants
cbuffer Transforms : register(b0)
{
	matrix world;
	matrix lightView;
	matrix lightProj;
};

cbuffer LightData : register(b1)
{
	float3 lightPos;
	float padding;
}

// I/O Structs
struct VSInput
{
	float3 pos : POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
};

struct VSOutput
{
	float4 lightViewPos : SV_POSITION;
	float3 lightPos : TEXCOORD2;
};

VSOutput main(VSInput input)
{
	VSOutput output;

	// calculate proj position of vertex as viewed from light source
	output.lightViewPos = mul(float4(input.pos, 1.0f), world);
	output.lightViewPos = mul(output.lightViewPos, lightView);
	output.lightViewPos = mul(output.lightViewPos, lightProj);

	// Calculate the position of the vertex in the world
	float4 worldPos = mul(float4(input.pos, 1.0f), world);

	// Output the world position of the vertex
	//output.pos = worldPos;

	// Determine the light position
	output.lightPos = lightPos.xyz - worldPos.xyz;

	// Normalize the light position vector
	output.lightPos = normalize(output.lightPos);

	return output;
}